#!/usr/bin/env python3

import argparse
from argparse import RawTextHelpFormatter
import os
import sys

parser = argparse.ArgumentParser(description="""Create fonts for LVGL including the built-in symbols. lv_font_conv needs to be installed. See https://github.com/lvgl/lv_font_conv
Example: python built_in_font_gen.py --size 16 -o lv_font_roboto_16.c --bpp 4 -r 0x20-0x7F""", formatter_class=RawTextHelpFormatter)
parser.add_argument('-s', '--size',
					type=int,
					metavar = 'px',
					nargs='?',
					help='Size of the font in px')
parser.add_argument('--bpp',
					type=int,
					metavar = '1,2,4',
					nargs='?',
					help='Bit per pixel')
parser.add_argument('-r', '--range',
					nargs='+',
					metavar = 'start-end',
					default=['0x20-0x7F,0xB0,0x2022'],
					help='Ranges and/or characters to include. Default is 0x20-7F (ASCII). E.g. -r 0x20-0x7F, 0x200, 324')
parser.add_argument('--symbols',
					nargs='+',
					metavar = 'sym',
					default=[''],
					help=u'Symbols to include. E.g. -s ÁÉŐ'.encode('utf-8'))
parser.add_argument('--font',
					metavar = 'file',
					nargs='?',
					default='Montserrat-Medium.ttf',
					help='A TTF or WOFF file')
parser.add_argument('-o', '--output',
					nargs='?',
					metavar='file',
					help='Output file name. E.g. my_font_20.c')
parser.add_argument('--compressed', action='store_true',
					help='Compress the bitmaps')
parser.add_argument('--subpx', action='store_true',
					help='3 times wider letters for sub pixel rendering')

args = parser.parse_args()

if args.compressed == False:
	compr = "--no-compress --no-prefilter"
else:
	compr = ""

if len(args.symbols[0]) != 0:
	args.symbols[0] = "--symbols " +  args.symbols[0]

subpx = ""
if args.subpx: subpx = "--lcd"

def parse_int(s: str) -> int:
    try:
        return int(s, 0)
    except ValueError:
        return int(s)

def symbol_filter_hex(input):
    if input != "":
        return "0x" + input.split()[1]
    else:
        return "";

symbols = '''network_wifi e1ba
network_wifi_1_bar ebe4
network_wifi_1_bar_locked f58f
network_wifi_2_bar ebd6
network_wifi_2_bar_locked f58e
network_wifi_3_bar ebe1
network_wifi_3_bar_locked f58d
network_wifi_locked f532
toolbar e9f7
desktop_windows e30c
cable efe6
bolt ea0b
battery_full_alt f13b
battery_horiz_000 f8ae
battery_horiz_050 f8af
battery_horiz_075 f8b0
calendar_clock f540
usb e1e0
usb_off e4fa
eject e8fb
lock e899
folder e2c7
draft e66d
docs ea7d
image e3f4
quick_reference e46e
screenshot f056
monitor_heart eaa2
close e5cd'''

symbols_split = symbols.split('\n')
symbols_hex_correct = map(symbol_filter_hex, symbols_split)
symbols_int = map(parse_int, symbols_hex_correct)
symbols_int_str = map(str, symbols_int)

syms = ",".join(symbols_int_str)

#Run the command (Add degree and bullet symbol)
cmd = "lv_font_conv {} {} --bpp {} --size {} --font {} -r {} {} --font MaterialSymbolsRounded.ttf -r {} --format lvgl -o {} --force-fast-kern-format".format(subpx, compr, args.bpp, args.size, args.font, args.range[0], args.symbols[0], syms, args.output)
os.system(cmd)

print("")
print("namespace tt::lvgl::fontsymbol {")
for line in symbols_split:
    parts = line.split();
    name = parts[0];
    value = ''.join('\\x{:02X}'.format(n) for n in chr(int(parts[1], 16)).encode('utf-8'))
    print("    constexpr const char* {} = \"{}\";".format(name, value))
print("} // namespace tt::lvgl")
print("")
